@@ -23,9 +23,9 @@ class.  For those wishing to override the braille generators, however,
 one can create a new instance and replace/extend the braille
 generators as they see fit."""
 
-__id__        = "$Id: braillegenerator.py 2708 2007-08-24 21:17:00Z eitani $"
-__version__   = "$Revision: 2708 $"
-__date__      = "$Date: 2007-08-24 17:17:00 -0400 (Fri, 24 Aug 2007) $"
+__id__        = "$Id: braillegenerator.py 2960 2007-10-01 22:40:22Z richb $"
+__version__   = "$Revision: 2960 $"
+__date__      = "$Date: 2007-10-01 18:40:22 -0400 (Mon, 01 Oct 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -107,6 +107,8 @@ class BrailleGenerator:
              self._getBrailleRegionsForPageTab
         self.brailleGenerators[rolenames.ROLE_PAGE_TAB_LIST]       = \
              self._getBrailleRegionsForPageTabList
+        self.brailleGenerators[rolenames.ROLE_PANEL]               = \
+             self._getBrailleRegionsForPanel
         self.brailleGenerators[rolenames.ROLE_PARAGRAPH]           = \
              self._getBrailleRegionsForText
         self.brailleGenerators[rolenames.ROLE_PASSWORD_TEXT]       = \
@@ -839,6 +841,40 @@ class BrailleGenerator:
 
         return self._getDefaultBrailleRegions(obj)
 
+    def _getBrailleRegionsForPanel(self, obj):
+        """Gets text to be displayed for a panel.
+
+        Arguments:
+        - obj: an Accessible
+
+        Returns a list where the first element is a list of Regions to
+        display and the second element is the Region which should get
+        focus.
+        """
+
+        self._debugGenerator("_getBrailleRegionsForPanel", obj)
+
+        regions = []
+
+        text = ""
+        text = self._script.appendString(
+            text, self._script.getDisplayedLabel(obj))
+
+        # If there was no label for the panel, but it has a name, we'll
+        # use the name.
+        #
+        if len(text) == 0:
+            text = self._script.appendString(
+                text, self._script.getDisplayedText(obj))
+
+        text = self._script.appendString(text, self._getTextForRole(obj))
+
+        regions = []
+        componentRegion = braille.Component(obj, text)
+        regions.append(componentRegion)
+
+        return [regions, componentRegion]
+
     def _getBrailleRegionsForProgressBar(self, obj):
         """Get the braille for a progress bar.  If the object already
         had focus, just the new value is displayed.
@@ -1086,21 +1122,6 @@ class BrailleGenerator:
 
         regions = []
 
-        # Check to see if this table cell contains an icon (image).
-        # If yes:
-        #   1/ Try to get a description for it and speak that.
-        #   2/ Treat the object of role type ROLE_IMAGE and speak
-        #      the role name.
-        # See bug #465989 for more details.
-        #
-        if obj.image:
-            if obj.image.imageDescription:
-                regions.append(obj.image.imageDescription)
-            obj.role = rolenames.ROLE_IMAGE
-            regions.extend(self._getBrailleRegionsForImage(obj))
-            obj.role = rolenames.ROLE_TABLE_CELL
-            return regions
-
         # If this table cell has 2 children and one of them has a
         # 'toggle' action and the other does not, then present this
         # as a checkbox where:
@@ -1150,7 +1171,7 @@ class BrailleGenerator:
                     obj.role = rolenames.ROLE_CHECK_BOX
                     regions = self._getBrailleRegionsForCheckBox(obj)
 
-                    # If this table cell doesn't have any label associated 
+                    # If this table cell doesn't have any label associated
                     # with it then also braille the table column header.
                     # See Orca bug #455230 for more details.
                     #
@@ -1175,6 +1196,23 @@ class BrailleGenerator:
                 self._script.getRealActiveDescendant(obj))
             regions[0].extend(cellRegions)
 
+        # Check to see if this table cell contains an icon (image).
+        # If yes:
+        #   1/ Try to get a description for it and speak that.
+        #   2/ Treat the object of role type ROLE_IMAGE and speak
+        #      the role name.
+        # See bug #465989 for more details.
+        #
+        displayedText = self._script.getDisplayedText( \
+                          self._script.getRealActiveDescendant(obj))
+        if (not displayedText or len(displayedText) == 0) and obj.image:
+            if obj.image.imageDescription:
+                regions[0].append(obj.image.imageDescription)
+            obj.role = rolenames.ROLE_IMAGE
+            [cellRegions, focusRegion] = self._getBrailleRegionsForImage(obj)
+            regions[0].extend(cellRegions)
+            obj.role = rolenames.ROLE_TABLE_CELL
+
         # [[[TODO: WDW - HACK attempt to determine if this is a node;
         # if so, describe its state.]]]
         #
@@ -1567,7 +1605,9 @@ class BrailleGenerator:
         parent = obj.parent
         if parent and parent.table:
             row = parent.table.getRowAtIndex(obj.index)
-            if row >= 0:
+            if (row >= 0) \
+                and (not obj.role in [rolenames.ROLE_ROW_HEADER,
+                                      rolenames.ROLE_TABLE_ROW_HEADER]):
                 desc = parent.table.getRowDescription(row)
             else:
                 desc = None
@@ -1586,7 +1626,9 @@ class BrailleGenerator:
                 regions.append(braille.Region(text))
 
             col = parent.table.getColumnAtIndex(obj.index)
-            if col >= 0:
+            if (col >= 0) \
+                and (not obj.role in [rolenames.ROLE_COLUMN_HEADER,
+                                      rolenames.ROLE_TABLE_COLUMN_HEADER]):
                 desc = parent.table.getColumnDescription(col)
             else:
                 desc = None
