@@ -23,9 +23,9 @@ class.  For those wishing to override the speech generators, however,
 one can create a new instance and replace/extend the speech generators
 as they see fit."""
 
-__id__        = "$Id: speechgenerator.py 2539 2007-07-13 18:32:02Z richb $"
-__version__   = "$Revision: 2539 $"
-__date__      = "$Date: 2007-07-13 14:32:02 -0400 (Fri, 13 Jul 2007) $"
+__id__        = "$Id: speechgenerator.py 2988 2007-10-07 15:42:30Z joanied $"
+__version__   = "$Revision: 2988 $"
+__date__      = "$Date: 2007-10-07 11:42:30 -0400 (Sun, 07 Oct 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -37,7 +37,9 @@ import orca_state
 import rolenames
 import settings
 
-from orca_i18n import _                          # for gettext support
+from orca_i18n import _         # for gettext support
+from orca_i18n import ngettext  # for ngettext support
+from orca_i18n import Q_        # to provide qualified translatable strings
 
 class SpeechGenerator:
     """Takes accessible objects and produces a string to speak for
@@ -94,6 +96,8 @@ class SpeechGenerator:
              self._getSpeechForLayeredPane
         self.speechGenerators[rolenames.ROLE_LIST]                = \
              self._getSpeechForList
+        self.speechGenerators[rolenames.ROLE_LIST_ITEM]           = \
+             self._getSpeechForListItem
         self.speechGenerators[rolenames.ROLE_MENU]                = \
              self._getSpeechForMenu
         self.speechGenerators[rolenames.ROLE_MENU_BAR]            = \
@@ -735,6 +739,38 @@ class SpeechGenerator:
 
         return utterances
 
+    def _getSpeechForListItem(self, obj, already_focused):
+        """Get the speech for a listitem.
+
+        Arguments:
+        - obj: the listitem
+        - already_focused: False if object just received focus
+
+        Returns a list of utterances to be spoken for the object.
+        """
+
+        utterances = self._getDefaultSpeech(obj, already_focused)
+
+        if obj.state.count(atspi.Accessibility.STATE_EXPANDABLE):
+            if obj.state.count(atspi.Accessibility.STATE_EXPANDED):
+                # Translators: this represents the state of a node in a tree.
+                # 'expanded' means the children are showing.
+                # 'collapsed' means the children are not showing.
+                #
+                utterances.append(_("expanded"))
+            else:
+                # Translators: this represents the state of a node in a tree.
+                # 'expanded' means the children are showing.
+                # 'collapsed' means the children are not showing.
+                #
+                utterances.append(_("collapsed"))
+                
+        self._debugGenerator("_getSpeechForListItem",
+                             obj,
+                             already_focused,
+                             utterances)
+        return utterances
+
     def _getSpeechForMenu(self, obj, already_focused):
         """Get the speech for a menu.
 
@@ -980,12 +1016,16 @@ class SpeechGenerator:
             # Translators: this is in reference to a radio button being
             # selected or not.
             #
-            selectionState = _("selected")
+            # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
+            #
+            selectionState = Q_("radiobutton|selected")
         else:
             # Translators: this is in reference to a radio button being
             # selected or not.
             #
-            selectionState = _("not selected")
+            # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
+            #
+            selectionState = Q_("radiobutton|not selected")
 
         # If it's not already focused, say it's name
         #
@@ -1124,7 +1164,6 @@ class SpeechGenerator:
         else:
             utterances = []
             utterances.extend(self._getSpeechForObjectLabel(obj))
-
             # Ignore the text on the slider.  See bug 340559
             # (http://bugzilla.gnome.org/show_bug.cgi?id=340559): the
             # implementors of the slider support decided to put in a
@@ -1137,7 +1176,6 @@ class SpeechGenerator:
             #
             if not utterances:
                 utterances.extend(self._getSpeechForObjectName(obj))
-
             utterances.extend(self._getSpeechForObjectRole(obj))
             utterances.append(valueString)
             utterances.extend(self._getSpeechForObjectAvailability(obj))
@@ -1213,7 +1251,7 @@ class SpeechGenerator:
         hasItems = False
         for i in range(0, obj.childCount):
             child = obj.child(i)
-            if child.state.count(atspi.Accessibility.STATE_SHOWING):
+            if child and child.state.count(atspi.Accessibility.STATE_SHOWING):
                 hasItems = True
                 break
         if not hasItems:
@@ -1289,12 +1327,24 @@ class SpeechGenerator:
                     obj.role = rolenames.ROLE_TABLE_CELL
                     break
 
-        utterances.append(self._script.getDisplayedText(\
-                          self._script.getRealActiveDescendant(obj)))
+        displayedText = self._script.getDisplayedText( \
+                          self._script.getRealActiveDescendant(obj))
+        utterances.append(displayedText)
 
-        # [[[TODO: WDW - HACK attempt to determine if this is a node;
-        # if so, describe its state.]]]
+        # If there is no displayed text, check to see if this table cell
+        # contains an icon (image). If yes:
+        #   1/ Try to get a description for it and speak that.
+        #   2/ Treat the object of role type ROLE_IMAGE and speak
+        #      the role name.
+        # See bug #465989 for more details.
         #
+        if (not displayedText or len(displayedText) == 0) and obj.image:
+            if obj.image.imageDescription:
+                utterances.append(obj.image.imageDescription)
+            obj.role = rolenames.ROLE_IMAGE
+            utterances.extend(self._getSpeechForImage(obj, already_focused))
+            obj.role = rolenames.ROLE_TABLE_CELL
+
         if obj.state.count(atspi.Accessibility.STATE_EXPANDABLE):
             if obj.state.count(atspi.Accessibility.STATE_EXPANDED):
                 # Translators: this represents the state of a node in a tree.
@@ -1302,6 +1352,19 @@ class SpeechGenerator:
                 # 'collapsed' means the children are not showing.
                 #
                 utterances.append(_("expanded"))
+                childNodes = self._script.getChildNodes(obj)
+                children = len(childNodes)
+
+                if not children \
+                   or (settings.speechVerbosityLevel == \
+                       settings.VERBOSITY_LEVEL_VERBOSE):
+                    # Translators: this is the number of items in a layered
+                    # pane or table.
+                    #
+                    itemString = ngettext("%d item",
+                                          "%d items",
+                                          children) % children
+                    utterances.append(itemString)
             else:
                 # Translators: this represents the state of a node in a tree.
                 # 'expanded' means the children are showing.
@@ -1309,15 +1372,6 @@ class SpeechGenerator:
                 #
                 utterances.append(_("collapsed"))
 
-            # If this is an expandable table cell with no children, then
-            # let the user know.
-            #
-            if obj.childCount == 0:
-                # Translators: this is the number of items in a layered pane
-                # or table.
-                #
-                utterances.append(_("0 items"))
-
         self._debugGenerator("_getSpeechForTableCell",
                              obj,
                              already_focused,
